@@ -1,11 +1,11 @@
 ;;; beanshell.el
-;; $Revision: 1.2 $ 
+;; $Revision: 1.4 $ 
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
 ;; Keywords: java, tools
 
-;; Copyright (C) 1997, 1998 Paul Kinnucan.
+;; Copyright (C) 1997, 1998, 2001 Paul Kinnucan.
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -22,21 +22,58 @@
 ;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 ;; Boston, MA 02111-1307, USA.
 
-(eval-when-compile
-  (require 'jde-run))
+;; Need jde-run only to get the definition for 
+;; save-w32-show-window macro.
 
-(defcustom bsh-startup-timeout 3
+
+(defcustom bsh-startup-timeout 10
   "*Length of time the JDE waits for the Beanshell to startup.
 Increase the value of this variable if you get Lisp errors
 on BeanShell startup on Unix."
   :group 'bsh
   :type 'integer)
 
+(defcustom bsh-eval-timeout 20
+  "*Length of time in seconds the JDE waits for the Beanshell to evaluate
+a Java expression before giving up and signaling an error."
+  :group 'bsh
+  :type 'integer)
+
+(defcustom bsh-vm-args nil
+  "*Specify arguments to be passed to the Java vm.
+This option allows you to specify one or more arguments to be passed
+to the Java vm that runs the BeanShell. Note that the value of this
+variable should be a list of strings, each of which represents an
+argument. When customizing this variable, use a separate text field
+for each argument."
+  :group 'jde-run-options
+  :type '(repeat (string :tag "Argument")))
+
+
 (defun bsh()
 "*Starts BeanShell, a Java interpreter developed by Pat Niemeyer."
   (interactive)
   (bsh-internal t))
 
+
+(eval-when-compile
+  (if (not (fboundp 'save-w32-show-window))
+      (defmacro save-w32-show-window (&rest body)
+	"Saves the value of the w32-start-process-show-window variable
+before evaluating body and restores the value afterwards."
+	`(if (and (eq system-type 'windows-nt)
+		  (not (string-match "XEmacs" (emacs-version))))
+	     (if (boundp 'win32-start-process-show-window)
+		 (let ((save win32-start-process-show-window))
+		   (setq win32-start-process-show-window t)
+		   ,@body
+		   (setq win32-start-process-show-window save))
+	       (let ((save w32-start-process-show-window))
+		 (setq w32-start-process-show-window t)
+		 ,@body
+		 (setq w32-start-process-show-window save)))	 
+	   ,@body))))
+
 (defun bsh-internal (&optional display-buffer) 
   (let ((bsh-buffer-name "*bsh*"))
     (if (not (comint-check-proc bsh-buffer-name))
@@ -51,24 +88,43 @@ on BeanShell startup on Unix."
 	       (vm-args
 		(list
 		 "-classpath"
-                 (jde-convert-cygwin-path
-                  (concat
-                   jde-java-directory "lib/jde.jar" jde-classpath-separator
-                   jde-java-directory "lib/bsh.jar" jde-classpath-separator
-                   (if jde-global-classpath
-                       (jde-run-build-classpath-arg jde-global-classpath)
-                     (getenv "CLASSPATH"))) jde-classpath-separator)
-		 "bsh.Interpreter")))
+		 (jde-build-classpath
+		  (append
+		   (list
+		    (expand-file-name "bsh-commands" jde-java-directory)
+		    (expand-file-name "lib/jde.jar" jde-java-directory)
+		    (expand-file-name "lib/bsh.jar" jde-java-directory))
+		   (if jde-global-classpath
+		       jde-global-classpath 
+		     (split-string (getenv "CLASSPATH") 
+				   jde-classpath-separator))))))
+	       (dir (if (buffer-file-name)
+			(file-name-directory (buffer-file-name))
+		      default-directory)))
+
+	  (setq vm-args (append vm-args bsh-vm-args))
+	  (setq vm-args (append vm-args (list "bsh.Interpreter")))
+
 	  (save-excursion
 	    (set-buffer bsh-buffer)
 	    (erase-buffer)
+
+	    (cd dir)
+	    (insert (concat "cd " dir "\n"))
+	    (insert 
+	     (concat vm " "
+		     (mapconcat (lambda (x) x) vm-args " ") "\n\n"))
+
 	    (comint-mode)
 	    (setq comint-prompt-regexp "bsh % "))
 	 (save-w32-show-window
 	   ;; (message "%s" (nth 1 vm-args))
 	   (message "%s" "Starting the BeanShell. Please wait...")
 	   (comint-exec bsh-buffer "bsh" vm nil vm-args))
-	 (process-kill-without-query (get-buffer-process bsh-buffer))
+
+	 (let ((bsh-process (get-buffer-process bsh-buffer)))
+	   (process-kill-without-query bsh-process))
+
 	 (if display-buffer
 	      (pop-to-buffer bsh-buffer-name)))
       (when display-buffer
@@ -80,10 +136,17 @@ on BeanShell startup on Unix."
 
 (defun bsh-eval-filter (process result)
   (let ((end-of-result (string-match ".*bsh % " result)))
+    ;; Check for case
+    ;;   %bsh\n...eval output...%bsh\n
+    ;; This can happen because the beanshell outputs two or more
+    ;; prompts after evaluating some expressions.
+    ;; Thanks to Stephane Nicolas.
+    ;; (if (eq end-of-result 0)
+    ;; (accept-process-output process 0 5))
     (if end-of-result
 	(setq bsh-tq-reply (concat bsh-tq-reply (substring result 0 end-of-result)))
       (setq bsh-tq-reply (concat bsh-tq-reply result))
-      (accept-process-output process 5 5))))
+      (accept-process-output process bsh-eval-timeout 5))))
 
 (defun bsh-eval (expr &optional eval-return)
   "Uses the BeanShell Java interpreter to evaluate a Java statement.
@@ -109,12 +172,28 @@ expression."
       (set-process-filter bsh-process 'bsh-eval-filter)
       ;; (message "Evaluating: %s" expr)
       (process-send-string bsh-process (concat expr "\n"))
-      (if (not (accept-process-output bsh-process 100 100))
-	  (message "No reply from BeanShell"))
+      (if (not (accept-process-output bsh-process bsh-eval-timeout 100))
+	  (error "No reply from BeanShell"))
       (set-process-filter bsh-process comint-filter)
+      (if (string-match "// Error:" bsh-tq-reply)
+	  (progn
+	    (message 
+	     "Beanshell expression evaluation error.\n  Expression: %s\n  Error: %s"
+	     expr bsh-tq-reply)
+	    (error "Beanshell eval error. See messages buffer for details.")))
       ;; (if eval-return (message "Evaluating reply: %s" bsh-tq-reply))
       (if eval-return
-	  (eval (read bsh-tq-reply))
+	  (if bsh-tq-reply
+	      (condition-case eval-error
+		  (eval (read bsh-tq-reply))
+		(error
+		 (message "Error evaluating Lisp result of Java expression evaluation.")
+		 (message "  Java expression: %s." expr)
+		 (message "  Java evaluation result: %s." bsh-tq-reply)
+		 (error "Error evaluating Java expresson. See *Messages* buffer.")))
+	    (progn
+	      (message "bsh-eval-r error: Beanshell result is null. Cannot evaluate.")
+	      (message "  Expression: %s" expr)))
 	bsh-tq-reply))))
 
 (defun bsh-eval-r(java-statement) 
@@ -129,11 +208,23 @@ t."
 (provide 'beanshell);
 
 ;; $Log: beanshell.el,v $
-;; Revision 1.2  2000/08/27 11:37:55  michaels
-;; Added require necessary for macros.
+;; Revision 1.4  2001/02/25 04:45:07  paulk
+;; Fixed bug in handling CLASSPATH environment variable.
 ;;
-;; Revision 1.1  2000/08/13 13:43:55  michaels
-;; Initial checkin.
+;; Revision 1.18  2001/02/03 07:44:26  paulk
+;; Now uses jde-build-classpath to build BeanShell classpath. This allows enviromnent variables in the classpath.
+;;
+;; Revision 1.17  2000/10/08 12:55:38  paulk
+;; *** empty log message ***
+;;
+;; Revision 1.16  2000/08/10 09:09:47  paulk
+;; Now handles Lisp eval errors gracefully.
+;;
+;; Revision 1.15  2000/08/07 05:11:38  paulk
+;; Adds bsh-vm-args variable.
+;;
+;; Revision 1.14  2000/08/04 02:51:19  paulk
+;; Added bsh-eval-timeout variable.
 ;;
 ;; Revision 1.13  2000/02/16 04:39:28  paulk
 ;; Implemented Cygwin/XEmacs compatiblity fix provided by Fred Hart
