@@ -1,11 +1,11 @@
 ;;; jde-db.el -- Debugger mode for jdb.
-;; $Revision: 1.2 $ $Date: 2000/10/06 08:51:20 $ 
+;; $Revision: 1.3 $ $Date: 2001/02/12 05:38:24 $ 
 
 ;; Author: Paul Kinnucan <paulk@mathworks.com>
 ;; Maintainer: Paul Kinnucan
 ;; Keywords: java, tools
 
-;; Copyright (C) 1997 Paul Kinnucan.
+;; Copyright (C) 1997, 2000, 2001 Paul Kinnucan.
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -41,22 +41,30 @@
 ;; jde-db variables
 
 (makunbound 'jde-db-debugger)
-(defcustom jde-db-debugger (cons "Other" (cons "jdb" "Executable"))
-"Specify the debugger you want to use to debug Java programs.
-Select JDEbug, if you want to use the JDE's builtin debugger. Otherwise,
-select Other Other may be an executable, like jdb, or a Java class. If Other 
-is a class, the JDE launches a vm to run the debugger. If you choose Other,
- select the debugger's type (executable or class) and enter the path name 
-of the debugger, if Other is an executable, or a  fully qualified package name,
-if Other is a class."
+(defcustom jde-db-debugger (cons "jdb" (cons "" "Executable"))
+"Specify the debugger you want to use to debug Java programs.  Select
+JDEbug, if you want to use the JDE's builtin debugger.  Select jdb, if
+you want to use the version of jdb that comes with JDK 1.3 (or later)
+or the Linux version of JDK 1.2.2.  Select oldjdb, if you want to use
+the version of jdb that comes with JDK 1.1.x (or earlier) versions of
+the JDK or with the Solaris and Windows versions of JDK 1.2.x.
+Otherwise, select Other. Other may be an executable or a Java
+class. If Other is a class, the JDE launches a vm to run the
+debugger. If you choose jdb or oldjdb, you must ensure that the jdb
+executable is in your system's command path.  If you choose Other,
+select the debugger's type (executable or class) and enter the path
+name of the debugger, if Other is an
+executable, or a fully qualified package name, if Other is a class."
   :group 'jde-project
   :type '(cons
 	  (radio-button-choice :format "%t \n%v"
 			       :tag "Debugger "
 			       (const "JDEbug")
+			       (const "jdb")
+			       (const "oldjdb")
 			       (const "Other"))
 	  (cons :tag "Other Debugger Info"
-	   (string :tag "Name")
+	   (file :tag "Path")
 	   (radio-button-choice :format "%t \n%v"
 			       :tag "Type "
 		 (const "Executable")
@@ -72,25 +80,16 @@ if Other is a class."
 	     (jde-get-java-source-buffers))
 	  (set-default sym val)))
 
+(makunbound 'jde-db-source-directories)
 (defcustom jde-db-source-directories nil
-  "*List of source directory paths.
-The JDE uses this list to locate source files corresponding
-to class files when debugging or building applications.
-When entering paths in the custom buffer, enter each path as a separate
-item in a separate edit field. Do NOT put more than one path in the
-same edit field. You'll only confuse JDE."
+  "*List of source directory paths.  The JDE uses this list to locate
+source files corresponding to class files when debugging or building
+applications.  When entering paths in the custom buffer, enter each
+path as a separate item in a separate edit field. Do NOT put more than
+one path in the same edit field. You'll only confuse JDE.
+Paths may contain environment variables."
   :group 'jde-project
-  :type '(repeat (string :tag "Path"))
-  :set '(lambda (sym val)
-	  (set-default 
-	   sym 
-	   (mapcar
-	    (lambda (path)
-	      (if (not (string= (substring path (- (length path) 1)) "/"))
-	       (concat path "/")
-	       path))
-	    val))))
-
+  :type '(repeat (file :tag "Path")))
 
 (defcustom jde-db-mode-hook nil
   "*Customization hook for jde-db inferior mode."
@@ -323,19 +322,15 @@ Run Option panel variables."
     ;; Set the classpath option. Use the local
     ;; classpath, if set; otherwise, the global
     ;; classpath.
-    (if jde-db-option-classpath
-	(setq options
-	      (nconc options 
-		     (list "-classpath"
-			   (jde-run-build-classpath-arg
-			    jde-db-option-classpath))))
-      (if jde-global-classpath
-	  (setq options
-		(nconc options
-		       (list "-classpath"
-			     (jde-run-build-classpath-arg
-			      jde-global-classpath))))))
-
+    (setq options
+	  (nconc
+	   options
+	   (list
+	    "-classpath"
+	   (jde-build-classpath
+	    (if jde-db-option-classpath
+		jde-db-option-classpath
+	      jde-global-classpath)))))
 
     ;; Set the verbose options.
     (let ((print-classes-loaded
@@ -478,8 +473,9 @@ Run Option panel variables."
 (defun jde-db-xemacs-menu ()
   (cons "Jdb" jde-db-xemacs-menu))
 
+;; (makunbound 'jde-db-marker-regexp)
 (defcustom jde-db-marker-regexp
-  "^Breakpoint hit: .*(\\([^\$]*\\).*:\\([0-9]*\\))"
+  "^.*: thread=.*, \\(\\(.*[.]\\)*\\)\\([^\$]*\\)\\(\$.*\\)*[.].+(), line=\\([0-9]*\\),"
 "*Regular expression used to find a jdb breakpoint position marker.
 The regular expression must have two subexpressions. The first matches
 the name of the class in which the breakpoint occurs; the second, the
@@ -487,8 +483,15 @@ line number at which the breakpoint occurs. The default expression
 matches breakpoint messages emitted by jdb. You may need to change
 the expression to accommodate other debuggers."
   :group 'jde-project
-  :type 'string
-)
+  :type 'string)
+
+(defvar jde-db-oldjdb-marker-regexp
+  "^Breakpoint hit: .*(\\([^\$]*\\).*:\\([0-9]*\\))"
+"*Regular expression used to find an oldjdb breakpoint position marker.
+The regular expression must have two subexpressions. The first matches
+the name of the class in which the breakpoint occurs; the second, the
+line number at which the breakpoint occurs. The default expression
+matches breakpoint messages emitted by oldjdb.")
 
 (defcustom jde-db-nodebug-marker-regexp
   "^Breakpoint hit: .*(pc \\([0-9]*\\))"
@@ -500,11 +503,11 @@ line number information is available.")
 ;; Thanks to "David J. Biesack" <sasdjb@unx.sas.com> for this function
 ;; and its use in jde-db-marker-filter.
 ;; Amended by "Patrick J. McNerthney" <pat@mcnerthney.com> to allow
-;; package names to begin underscores.
+;; package names to begin with underscores.
 (defun jde-db-make-qualified-class-name-regexp (class)
-"Constructs a regular expression to extract a qualified class name from a jdb
+  "Constructs a regular expression to extract a qualified class name from a jdb
 breakpoint message."
-(concat "\\(\\(\\(\\(\\w\\|[_]\\)*\\.\\)*\\)" class "\\)\\(\\b\\|\\$\\)"))
+  (concat "\\(\\(\\(\\(\\w\\|[_]\\)*\\.\\)*\\)" class "\\)\\(\\b\\|\\$\\)"))
 
 ;; David Biesack's original expression.
 ;;(concat "\\b\\(\\(\\(\\(\\w\\|[_]\\)*\\.\\)*\\)" class "\\)\\(\\b\\|\\$\\)"))
@@ -518,6 +521,10 @@ breakpoint message."
 (defvar jde-db-marker-acc "")
 (make-variable-buffer-local 'jde-db-marker-acc)
 
+(defvar jde-db-stack-depth "")
+(make-variable-buffer-local 'jde-db-stack-depth)
+
+
 (defun jde-db-marker-filter (input)
 
   ;; Accumulate next chunk of debugger output.
@@ -531,20 +538,30 @@ breakpoint message."
 		    (substring jde-db-marker-acc 0 (match-beginning 0))
 		    (substring jde-db-marker-acc (match-end 0)))))
 		    
-  (let ((output ""))
+  (let* ((output "")
+	 (oldjdbp (if (string= (car jde-db-debugger) "oldjdb") t))
+	 (marker-regexp 
+	  (if oldjdbp jde-db-oldjdb-marker-regexp jde-db-marker-regexp))
+	 (marker-regexp-class-index (if oldjdbp 1 3))
+	 (marker-regexp-line-index (if oldjdbp 2 5)))
 
     ;; (message (concat "jdb output:" input))
     ;; (message (concat "acc = " jde-db-marker-acc))
     
     ;; Process all the complete markers in this chunk.
-    (if (string-match jde-db-marker-regexp jde-db-marker-acc)
+    (if (string-match marker-regexp jde-db-marker-acc)
 	;; Extract the frame position from the marker.
 	(let ((premarker (substring jde-db-marker-acc 0 (match-beginning 0)))
 	      (marker (substring jde-db-marker-acc (match-beginning 0) (match-end 0)))
-	      (class (substring jde-db-marker-acc  (match-beginning 1) (match-end 1)))
-	      (line-no (string-to-int (substring jde-db-marker-acc
-						 (match-beginning 2)
-						 (match-end 2))))
+	      (class (substring 
+		      jde-db-marker-acc  
+		      (match-beginning marker-regexp-class-index) 
+		      (match-end marker-regexp-class-index)))
+	      (line-no (string-to-int 
+			(substring 
+			 jde-db-marker-acc
+			 (match-beginning marker-regexp-line-index)
+			 (match-end marker-regexp-line-index))))
 	      (rest (substring jde-db-marker-acc (match-end 0))))
 
 	  (setq gud-last-frame (cons (concat class ".java") line-no))
@@ -560,12 +577,11 @@ breakpoint message."
 	    ;;(message "case-fold-search = %s" (if case-fold-search "true" "false"))
 	    )
 
-	  (setq output (concat premarker "Stopped at line " (int-to-string line-no) " in "
-			       class ".java"))
+	  ;; Insert debugger output into debugger buffer.
+	  (setq output (concat premarker marker))
 
 	  ;; Set the accumulator to the remaining text.
-	  (setq jde-db-marker-acc rest)
-	  ))
+	  (setq jde-db-marker-acc rest)))
 
     ;; Handle case where there is no line number info in current class.
     (if (string-match jde-db-nodebug-marker-regexp jde-db-marker-acc) 
@@ -574,15 +590,59 @@ breakpoint message."
 	      (pc (substring jde-db-marker-acc (match-beginning 1) (match-end 1)))
 	      (rest (substring jde-db-marker-acc (match-end 0))))
 
-	  (setq output (concat premarker marker))
-	  ))
+	  (setq output (concat premarker marker))))
+
+    ;; Thanks to Michael Ernst <mernst@cs.washington.edu> for the following
+    ;; stack-related code.
+    ;;
+    ;; Extract the index of the current stack frame from the jdb prompt, where
+    ;; the prompt is of the form 
+    ;;
+    ;;   thread[stack_index]
+    ;; 
+    ;; e.g.,
+    ;;
+    ;;   main[1]
+    ;;
+    ;; The user can move the debugger up and down the stack via the up and
+    ;; down commands. The debugger indicates the current location by the
+    ;; stack index portion of its prompt.
+    (if (string-match "^[-a-zA-Z0-9_$]+\\[\\([0-9]+\\)\\] " jde-db-marker-acc)
+	(setq jde-db-stack-depth (match-string 1 jde-db-marker-acc)))
+
+    ;; The following form processes the output of the jdb where
+    ;; command, which lists the current stack. An example of the output
+    ;; is
+    ;;
+    ;;   [1] jmath.LinearSystem$InnerClass.print (LinearSystem.java:36)
+    ;;   [2] jmath.LinearSystem.<init> (LinearSystem.java:52)
+    ;;   [3] jmath.Test.main (Test.java:38)
+    ;;
+    ;; The form positions the source line cursor at the position that
+    ;; matches the current location of the debugger in the program's
+    ;; stack (set by the jdb up and down stack commands).
+    (if (string-match (concat "^  \\[" jde-db-stack-depth
+			      "\\] .*(\\([^\$\n]*\\).*:\\([0-9]+\\))")
+		      jde-db-marker-acc)
+	(let ((marker (match-string 0 jde-db-marker-acc))
+	      (class (match-string 1 jde-db-marker-acc))
+	      (line-no (string-to-int (match-string 2 jde-db-marker-acc))))
+	  (if (equal ".java" (substring class -5))
+	      (setq class (substring class 0 -5)))
+	  (setq gud-last-frame (cons (concat class ".java") line-no))
+	  ;; Extract package path from input.
+	  (setq jde-db-last-package "")
+	  (let ((case-fold-search nil)) ;; Make sure search is case-sensitive
+	    (and (string-match (jde-db-make-qualified-class-name-regexp class) marker)
+		 (setq jde-db-last-package
+		       (substring marker (match-beginning 2) (match-end 2)))))))
 
     ;; Does the remaining text look like it might end with the
     ;; beginning of another marker?  If it does, then keep it in
     ;; jde-db-marker-acc until we receive the rest of it.  Since we
     ;; know the full marker regexp above failed, it's pretty simple to
     ;; test for marker starts.
-    (if (string-match "^Breakpoint hit:" jde-db-marker-acc)
+    (if (string-match "\\(^Breakpoint hit:\\)\\|\\(^Step completed:\\)" jde-db-marker-acc)
 	(progn
 	  ;; Everything before the potential marker start can be output.
 	  (setq output (concat output (substring jde-db-marker-acc
@@ -630,20 +690,17 @@ breakpoint message."
     (let ((len (length jde-db-source-directories))
 	  (n 0))
       (while (< n len)
-	(let ((curr-dir (elt jde-db-source-directories n)))
+	(let ((curr-dir 
+	       (substitute-in-file-name (elt jde-db-source-directories n))))
 	  (cond
 	   ((jde-db-contains-file-p curr-dir file)
-	    ;(message "jde-db-search-src-dirs found %s in %s" file curr-dir)
 	    (throw 'found curr-dir))
 	   ((and (jde-db-contains-package-p curr-dir package)
 		 (jde-db-contains-file-p
-		  (concat curr-dir (jde-db-pkg-to-path package)) file))
-	    ;; (message "jde-db-search-src-dirs found %s in %s" file (concat curr-dir (jde-db-pkg-to-path jde-db-last-package)))
+		  (expand-file-name (jde-db-pkg-to-path package) curr-dir) file))
 	    (throw 'found
-		   (concat curr-dir 
-			   (jde-db-pkg-to-path package))))
-	   ;; (t (message "jde-db-search-src-dirs: %s not in %s" file curr-dir))
-	    ))
+		   (expand-file-name  
+			   (jde-db-pkg-to-path package) curr-dir)))))
 	(setq n (1+ n))))))
 
 ;; Fixes a bug in gud-make-debug-menu
@@ -681,7 +738,8 @@ and create such a submap in that new map."
 		  (local-set-key [menu-bar debug refresh] nil)
 		  (local-set-key [menu-bar debug stepi] nil)
 		  (local-set-key [menu-bar debug up] '("Up Stack" . gud-up))
-		  (local-set-key [menu-bar debug down] '("Down Stack" . gud-down))))
+		  (local-set-key [menu-bar debug down] '("Down Stack" . gud-down))
+		  (local-set-key [menu-bar debug where] '("Where" . gud-where))))
 	      buf)))
 	  (message (concat "Error: could not find %s. "
 			   "See jde-db-source-directories.") f)
@@ -846,23 +904,32 @@ we're in the jdb buffer)."
 			     jde-run-working-directory)))
     (if (not (comint-check-proc debug-buf-name))
 	(let* ((debug-buffer (get-buffer-create debug-buf-name))
-	       (program (if (string= (cdr (cdr jde-db-debugger)) "Executable")
-			   (car (cdr jde-db-debugger))
-			 jde-run-java-vm))
-	       (prog-args (if (string= (cdr (cdr jde-db-debugger)) "Executable")
-			      (append 
-			       (jde-db-get-vm-args)
-			       (jde-db-get-vm-args-from-user)
-			       (list app-class)
-			       jde-db-option-application-args
-			       (jde-db-get-app-args-from-user))
-			    (append
-			     (list (car (cdr jde-db-debugger)))
-			     (jde-db-get-vm-args)
-			     (jde-db-get-vm-args-from-user)
-			     (list app-class)
-			     jde-db-option-application-args
-			     (jde-db-get-app-args-from-user))))
+	       (debugger (car jde-db-debugger))
+	       (program 
+		(cond
+		 ((string= debugger "jdb") "jdb")
+		 ((string= debugger "oldjdb") "jdb")
+		 ((string= debugger "Other")
+		  (if (string= (cdr (cdr jde-db-debugger)) "Executable")
+		      (car (cdr jde-db-debugger))
+		    jde-run-java-vm))
+		 (t
+		  (error "Unknown debugger: %s" debugger))))
+	       (prog-args
+		(let ((debugger-args
+		       (append 
+			(jde-db-get-vm-args)
+			(jde-db-get-vm-args-from-user)
+			(list app-class)
+			jde-db-option-application-args
+			(jde-db-get-app-args-from-user))))
+		      (if (string= program "jdb")
+			  debugger-args
+			(if (string= (cdr (cdr jde-db-debugger)) "Executable")
+			    debugger-args
+			  (append
+			   (list (car (cdr jde-db-debugger)))
+			   debugger-args)))))
 	       (command-string (concat
 				program " "
 				(jde-run-make-arg-string prog-args) "\n\n")))
@@ -890,8 +957,7 @@ we're in the jdb buffer)."
 	    (mapc 'gud-basic-call jde-db-startup-commands)
 	   (when jde-db-set-initial-breakpoint
 	    (gud-basic-call (concat "stop in " app-class ".main"))
-	    (gud-basic-call "run")))
-	  )
+	    (gud-basic-call "run"))))
       (message "An instance of %s is running." app-class)			
       (pop-to-buffer debug-buf-name))))			   
 	  
@@ -941,9 +1007,11 @@ CLASS is the name of the class you are debugging."
 	 (not app-class)
 	 (string= app-class ""))
 	(setq app-class
+	      (if (buffer-file-name)
 	      (concat (jde-db-get-package)
 		      (file-name-sans-extension 
-		       (file-name-nondirectory (buffer-file-name))))))
+			   (file-name-nondirectory (buffer-file-name))))
+		(read-string "Java class to debug: "))))
     (jde-db-init app-class 'jde-db-marker-filter 'jde-db-find-file))
 
   (jde-db-def gud-break  "stop at %c:%l"  "\C-b" "Set breakpoint at current line.")
@@ -951,17 +1019,18 @@ CLASS is the name of the class you are debugging."
   (jde-db-def gud-step   "step"           "\C-s" "Step one source line with display.")
   (jde-db-def gud-next   "next"           "\C-n" "Step one line (skip functions).")
   (jde-db-def gud-cont   "cont"           "\C-r" "Continue with display.")
-  (jde-db-def gud-up     "up %p"          "<" "Up N stack frames (numeric arg).")
-  (jde-db-def gud-down   "down %p"        ">" "Down N stack frames (numeric arg).")
+  (jde-db-def gud-up     "up %p"          "<"    "Up N stack frames (numeric arg).")
+  (jde-db-def gud-down   "down %p"        ">"    "Down N stack frames (numeric arg).")
+  (jde-db-def gud-where  "where"          "\C-w" "Display location on the stack.")
   (jde-db-def gud-print  "print %e"       "\C-p" "Print object.")
-  (local-set-key [menu-bar debug up]   '("Up Stack" . gud-up))
-  (local-set-key [menu-bar debug down] '("Down Stack" . gud-down))
+  ;; (local-set-key [menu-bar debug up]   '("Up Stack" . gud-up))
+  ;; (local-set-key [menu-bar debug down] '("Down Stack" . gud-down))
+  ;; (local-set-key [menu-bar debug where] '("Where" . gud-where))
   (make-local-variable 'comint-prompt-regexp)
-  (setq comint-prompt-regexp "\\(^> *\\)\\|\\(^.*\\[[0-9]*\\] *\\)")
+  (setq comint-prompt-regexp "\\(^> *\\)\\|\\(^.*\\[[0-9]+\\] *\\)")
   (make-local-variable 'paragraph-start)
   (setq paragraph-start comint-prompt-regexp)
-  (run-hooks 'jde-db-mode-hook)
-  )
+  (run-hooks 'jde-db-mode-hook))
 
 (defun check-source-path (path) 
   "Return a valid path or nil if path is not found."  
@@ -973,42 +1042,16 @@ CLASS is the name of the class you are debugging."
 	p
       nil)))
 
-;;;###autoload
-(defun jde-db-set-source-paths (paths)
- "Set the source path list used by JDE to locate Java source files
-visited by the debugger. PATHS is a list of source paths separated by
-colons or semicolons, depending on the operating system.. Note that
-for packages, you need enter only the directory containing the
-top-level package. For example, including '../jdk1.1/src/' enables the
-JDE to locate all source files provided with JDK1.1. Note also that
-the paths must end in a slash."
- (interactive
-  "sEnter source paths: ")
- (let ((m 0)
-       (n (string-match jde-classpath-separator paths)))
-   (setq jde-db-source-directories (list))
-   (while n
-     (let ((path (check-source-path (substring paths m n))))
-       (if path
-	   (setq jde-db-source-directories
-		 (cons path jde-db-source-directories)))
-       (setq m (+ n 1))
-       (setq n (string-match jde-classpath-separator paths m))))
-   (setq n (length paths))
-   (if (and (> n 0) (< m n))
-       (let ((path (check-source-path (substring paths m n))))
-	 (if path
-	     (setq jde-db-source-directories
-		   (cons path jde-db-source-directories)))))
-   (setq jde-db-source-directories (nreverse jde-db-source-directories))))
 
-
-(defun jde-db-applet-init(applet-class marker-filter find-file applet-doc)
-  (let ((debug-buf-name (concat "*debug" applet-class "*"))
-	(source-directory default-directory)
-	(working-directory (if (string= jde-run-working-directory "")
-			       default-directory
-			     jde-run-working-directory)))
+(defun jde-db-applet-init (applet-class marker-filter find-file applet-doc-path)
+  (let* ((debug-buf-name (concat "*debug-" applet-class "*"))
+	 (applet-doc (file-name-nondirectory applet-doc-path))
+	 (applet-doc-directory (file-name-directory applet-doc-path))
+	 (source-directory default-directory)
+	 (working-directory 
+	  (if applet-doc-directory
+	      applet-doc-directory
+	    source-directory)))
     (if (not (comint-check-proc debug-buf-name))
 	(let* ((debug-buffer (get-buffer-create debug-buf-name))
 	       (program "appletviewer")
@@ -1075,7 +1118,7 @@ the paths must end in a slash."
   (local-set-key [menu-bar debug up]   '("Up Stack" . gud-up))
   (local-set-key [menu-bar debug down] '("Down Stack" . gud-down))
   (make-local-variable 'comint-prompt-regexp)
-  (setq comint-prompt-regexp "\\(^> *\\)\\|\\(^.*\\[[0-9]*\\] *\\)")
+  (setq comint-prompt-regexp "\\(^> *\\)\\|\\(^.*\\[[0-9]+\\] *\\)")
   (make-local-variable 'paragraph-start)
   (setq paragraph-start comint-prompt-regexp)
   (run-hooks 'jde-db-mode-hook)
@@ -1096,17 +1139,17 @@ sets a breakpoint in the init method of the class specified by
 file in the current buffer."
   (interactive
    (let ((insert-default-directory nil))
-     (list (read-file-name "Applet doc: " nil nil nil jde-run-applet-document))))
-  (setq jde-run-applet-document doc)
-  (let ((applet-doc (if (and jde-run-applet-document
-			     (not (string= jde-run-applet-document "")))
-			jde-run-applet-document
-		      (if (and jde-run-applet-doc
-			       (not (string= jde-run-applet-doc "")))
-			    jde-run-applet-doc
-			  (car (jde-run-find-html-files))))))
-    (if applet-doc 
-	(jde-db-applet-internal applet-doc) 
+     (list (read-file-name "Applet doc: " nil nil nil jde-run-applet-last-doc))))
+  (setq jde-run-applet-last-doc doc)
+  (let ((applet-doc-path 
+	 (if doc 
+	     doc
+	   (if (and jde-run-applet-doc
+		    (not (string= jde-run-applet-doc "")))
+	       jde-run-applet-doc
+	     (car (jde-run-find-html-files))))))
+    (if applet-doc-path 
+	(jde-db-applet-internal applet-doc-path) 
       (signal 'error "Could not find html document to display applet."))))
 
 
@@ -1119,11 +1162,28 @@ file in the current buffer."
 
 ;; Change History
 ;; $Log: jde-db.el,v $
-;; Revision 1.2  2000/10/06 08:51:20  youngs
-;; Martin's Monster Mega typo patch
+;; Revision 1.3  2001/02/12 05:38:24  paulk
+;; JDE 2.2.7
 ;;
-;; Revision 1.1  2000/08/13 13:43:58  michaels
-;; Initial checkin.
+;; Revision 1.69  2001/02/03 08:44:56  paulk
+;; Changed declaration of customized variables to allow path completion.
+;; Now allows environment variables in jde-db-source-directories.
+;;
+;; Revision 1.68  2001/02/03 07:28:06  paulk
+;; Now uses generalized jde-build-classpath function to build classpath argument to debugger.
+;;
+;; Revision 1.67  2000/12/18 05:22:45  paulk
+;; *** empty log message ***
+;;
+;; Revision 1.66  2000/10/10 06:36:59  paulk
+;; Fixed bug where selecting Other and Executable as the debugger results in the executable name being inserted twice.
+;;
+;; Revision 1.65  2000/10/08 12:55:39  paulk
+;; *** empty log message ***
+;;
+;; Revision 1.64  2000/08/19 06:46:22  paulk
+;; Updated to handle JDK 1.3 version of jdb. Source pointer now moves to
+;; current location in current stack frame.
 ;;
 ;; Revision 1.63  2000/06/12 08:37:43  paulk
 ;; Now displays JDEbug menu when running XEmacs.
@@ -1377,4 +1437,4 @@ file in the current buffer."
 ;; Initial revision
 ;;
 
-;;; end of jde-db.el
+;;; end of jde-db.el
\ No newline at end of file
